package domus;

import java.io.IOException;
import java.util.Calendar;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;

import domus.mensajes.SensorsMsg;
import domus.mensajes.TimerMsg;
import domus.mensajes.TempMsg;
import domus.mensajes.PhotoMsg;
import domus.mensajes.HallMsg;
import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

import javax.swing.*;

public class TinyDomus extends JPanel implements MessageListener {

	private static final long serialVersionUID = 1;
	private final int SENSOR_LUZ = 1;
	private final int SENSOR_TEMP = 2;
	private final static int RETARDO = 800;
    private  final static String FORMATO_FECHA_HORA = "dd/MM/yyyy HH:mm:ss";
	private final boolean DEBUG = false;
	private static ControlRemoto mando= null;
	public static TinyDomus mensajes = null;
	
	private int photoMin = 0;
	private int tempMin = 0;
	private int tempMax = 0;
	private int timer = 0;
	
	private MoteIF mif;
	private MoteIF mif2;
		
	public static void main (String args[]) throws Exception{
		
		//Solo permitimos la ejecucion de una instancia de TinyDomus
		if (TinyDomus.mensajes != null) {
			System.err.println("ERROR: Aplicaci�n ya en uso");
			System.exit(0);
		} else {
			//Activamos el entorno gráfico
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					ControlRemoto.createAndShowGUI();
					mando = ControlRemoto.getContentPane();
				}	   
			});
				
			//Activamos el gestor de mensajes
			mensajes = new TinyDomus();
			PhoenixSource phoenix;
			phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		
			String source="sf@localhost:9002";	
	    	phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
	    	mensajes.mif = new MoteIF(phoenix);
			mensajes.mif2 = new MoteIF(phoenix);
			mensajes.mif.registerListener(new SensorsMsg(), mensajes);
			mensajes.mif.registerListener(new TimerMsg(), mensajes);
			mensajes.mif.registerListener(new TempMsg(), mensajes);
			mensajes.mif.registerListener(new PhotoMsg(), mensajes);
			mensajes.mif.registerListener(new HallMsg(), mensajes);
			
		}	
	}	
	
	//Gestión de los mensajes recibidos
	@Override
	public void messageReceived(int to, Message m) {
		String men = null;
		if(m instanceof SensorsMsg){
			SensorsMsg sm = (SensorsMsg) m;
			double lecturaVoltsBat = passarAVoltios(sm.get_vrefmilivolts(),sm.get_countsBat())*2;	//4 es el factor d'atenuacio del divisor
			double lecturaVoltsTemp = passarAVoltios(sm.get_vrefmilivolts(),sm.get_countsTemp());
			double temperaturaCentigrados = (lecturaVoltsTemp-0.5)/0.01;
			if (sm.get_tipo() == SENSOR_LUZ) {
			   men = "     MENSAJE SENSORSMSG-> Tipo: CONSULTA FOTOSENSOR"+"\tVCC:"+formatearDouble(lecturaVoltsBat)+"\tC_PHOTO:"+sm.get_countsPhoto();	
			   if (DEBUG) System.out.println(men);
			   mando.actualizarValorPhoto(sm.get_countsPhoto(), photoMin);
			} else if(sm.get_tipo() == SENSOR_TEMP) {
			   men = "     MENSAJE SENSORSMSG-> Tipo: CONSULTA TEMPERATURA"+"\tVCC:"+formatearDouble(lecturaVoltsBat)+"\tTEMP:"+formatearDouble(temperaturaCentigrados);	
			   if (DEBUG) System.out.println(men);	
			   mando.actualizarValorTemp(temperaturaCentigrados, tempMin, tempMax);
			} else { 
			   men = "     MENSAJE SENSORSMSG-> Tipo: CONSULTA LUZ/TEMPERATURA"+"\tVCC:"+formatearDouble(lecturaVoltsBat)+"\tC_PHOTO:"+sm.get_countsPhoto()+"\tTEMP:"+formatearDouble(temperaturaCentigrados);	
			   if (DEBUG) System.out.println(men);	
			   mando.actualizarValorPhoto(sm.get_countsPhoto(), photoMin);
			   mando.actualizarValorTemp(temperaturaCentigrados, tempMin, tempMax);
			}
			mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		}
		else if(m instanceof TimerMsg){
			TimerMsg tim =  (TimerMsg) m;
			if (tim.get_consulta() == 1) {
				timer = ((int)tim.get_tempor()/1000);
				men = " MENSAJE TIMERMSG-> Tipo: CONSULTA VALOR. " + "Mota_ID "+tim.get_moteId()+
		         ". Intervalo muestreo: "+ timer + " segundos.";
				if (DEBUG) System.out.println(men);
			} else {
				if (tim.get_tempor() == 0) {
					timer = 0;
					men = " MENSAJE TIMERMSG-> Tipo: PARADA MUESTREO. ";
					if (DEBUG) System.out.println(men);
				} else {
					timer = ((int)tim.get_tempor()/1000);
					men = " MENSAJE TIMERMSG-> Tipo: MODIFICAR INTERVALO. " + "Mota_ID " + 
					tim.get_moteId() + ". Intervalo muestreo: " + timer + " segundos.";
					if (DEBUG) System.out.println(men);
				}
			}
			mando.actualizarTimer(timer);
			mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		}
		else if(m instanceof PhotoMsg){
			PhotoMsg pho = (PhotoMsg) m;
			photoMin = pho.get_photoMin();
			if (pho.get_consulta() == 1) {
				men = "  MENSAJE PHOTOMSG-> Tipo: CONSULTA VALOR MINIMO. " +
				   "Mota_ID "+pho.get_moteId() + ". Valor MIN: "+ photoMin + ".";
				if (DEBUG) System.out.println(men);
			} else {
				men = "  MENSAJE PHOTOMSG-> Tipo: MODIFICAR VALOR MINIMO. " +
				   "Mota_ID "+pho.get_moteId() + ". Valor MIN: "+ photoMin + ".";
				if (DEBUG) System.out.println(men);
			}
			mando.actualizarPhotoMin(photoMin);
			mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		}	
		else if(m instanceof TempMsg){
		    TempMsg tem = (TempMsg) m;
		    tempMin = tem.get_tempMin();
		    tempMax = tem.get_tempMax();
		    if (tem.get_consulta() == 1) {
		    	men = "   MENSAJE TEMPMSG-> Tipo: CONSULTA VALORES MIN/MAX. " +
				   "Mota_ID "+tem.get_moteId() + ". Valor MIN: "+ tempMin +
				   ". Valor MAX: "+ tempMax + ".";
		    	if (DEBUG) System.out.println(men);
		    } else {
		    	men = "   MENSAJE TEMPMSG-> Tipo: MODIFICAR VALORES MIN/MAX. " +
				   "Mota_ID "+tem.get_moteId() + ". Valor MIN: "+ tempMin +
				   ". Valor MAX: "+ tempMax + ".";
		    	if (DEBUG) System.out.println(men);
		    }
		    mando.actualizarTempMin(tempMin);
		    mando.actualizarTempMax(tempMax);
		    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		}
		else if (m instanceof HallMsg) {
			HallMsg ehall = (HallMsg) m;
			men = "\n    MENSAJE HALLMSG-> Tipo: ACTIVAR SENSOR HALL. " +
			   "Mota_ID "+ehall.get_moteId() + ". Estado: "+ehall.get_hallOn() + ".";
			if (DEBUG) System.out.println(men);  
		    mando.actualizarHall("On");
		    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		    //Con la respuesta, la mota remota debe apagar el led2
		    try {
		      Thread.sleep(RETARDO);
		    } catch (InterruptedException ie) {}
		    men = "Envio HALLMSG: Respuesta confirmacion mensaje Efecto HALL.";
		    if (DEBUG) System.out.println(men);
		    HallMsg ehmsg = new HallMsg();
		    ehmsg.set_moteId(0);
		    ehmsg.set_hallOn((byte)0);
		    try {
			   //65535 emite en broadcast para todas las motas
			   mif2.send(65535,ehmsg);
		    } catch (IOException e) {
			  System.err.println("No se ha podido en enviar el mensaje.");
		    }
		    mando.actualizarHall("Off");
		    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
		}
		else{
			System.err.println("Se ha recibido un mensaje desconocido, el amType es "+m.amType());
		}
	}
	
	private double passarAVoltios(int vrefMilivolts, int counts){
		return (((double)counts)/1024)*(((double)2405)/1000);
	}
	
	private String formatearDouble(double d){
		DecimalFormat df = new DecimalFormat("#0.00");
		return df.format(d);
	}
	
	public void consultarConfigTimer() {
		String men = "Send TIMERMSG: Consulta valor temporizador.";
		TimerMsg timsg = new TimerMsg();
	    timsg.set_moteId((int) 0);
	    timsg.set_tempor(0);
	    timsg.set_consulta((byte)1);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,timsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }	
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para consultar parámetros MIN/MAX temperatura
	public void consultarConfigTemperatura() {
		String men = "Send TEMPMSG: Consulta valor MIN/MAX temperatura.";
		TempMsg temsg = new TempMsg();
	    temsg.set_moteId((short) 0);
	    temsg.set_tempMin((short)0);
	    temsg.set_tempMax((short)0);
	    temsg.set_consulta((byte)1);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,temsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje consulta valor MIN fotosensor
	public void consultarConfigFotosensor() {
		String men = "Send PHOTOMSG: Consulta valor MIN fotosensor.";
		PhotoMsg phmsg = new PhotoMsg();
	    phmsg.set_moteId((short) 0);
	    phmsg.set_photoMin((short)0);
	    phmsg.set_consulta((byte)1);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,phmsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para modificar el valor de temporizador
	public void modificarConfigTimer(int timer) {
		int miliseg = (int)(timer*1000);
		String men = "Send TIMERMSG: Modificar valor temporizador (valor " + timer + " seg).";
		TimerMsg timsg = new TimerMsg();
	    timsg.set_moteId((short) 0);
	    timsg.set_tempor(miliseg);
	    timsg.set_consulta((byte)0);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,timsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para modificar valores MIN/MAX sensor temperatura
	public void modificarConfigTemperatura(short min, short max) {
		String men = "Send TEMPMSG: Modificar valor MIN/MAX temperatura (MIN " + min + " - MAX " + max + ").";
		TempMsg temsg = new TempMsg();
	    temsg.set_moteId((short) 0);
	    temsg.set_tempMin(min);
	    temsg.set_tempMax(max);
	    temsg.set_consulta((byte)0);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,temsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para modficar el valor MIN del fotosensor
	public void modificarConfigFotosensor(short timer) {
		String men = "Send PHOTOMSG: Modificar valor MIN fotosensor (valor " + timer + ").";
		PhotoMsg phmsg = new PhotoMsg();
	    phmsg.set_moteId((short) 0);
	    phmsg.set_photoMin(timer);
	    phmsg.set_consulta((byte)0);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,phmsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para consultar el sensor de temperatura
	public void consultarSensorTemperatura() {
		String men = "Send SENSORSMSG: Consulta sensor TEMPERATURA.";
		SensorsMsg semsg = new SensorsMsg();
	    semsg.set_moteId(0);
	    semsg.set_countsBat(0);
	    semsg.set_vrefmilivolts(0);
	    semsg.set_countsPhoto(0);
	    semsg.set_countsTemp(0);
	    semsg.set_tipo((short)2);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,semsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Envío mensaje para consultar el vamor del fotosensor
	public void consultarFotosensor() {
		String men = "Send SENSORSMSG: Consulta sensor LUMINOSIDAD.";
		SensorsMsg semsg = new SensorsMsg();
	    semsg.set_moteId(0);
	    semsg.set_countsBat(0);
	    semsg.set_vrefmilivolts(0);
	    semsg.set_countsPhoto(0);
	    semsg.set_countsTemp(0);
	    semsg.set_tipo((short)1);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,semsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }	
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Mensaje para consultar ambos sensores: temperatura y luminosidad
	public void consultarAmbos() {
		String men = "Send SENSORSMSG: Consulta sensores TEMPERATURA y LUMINOSIDAD.";
		SensorsMsg semsg = new SensorsMsg();
	    semsg.set_moteId(0);
	    semsg.set_countsBat(0);
	    semsg.set_vrefmilivolts(0);
	    semsg.set_countsPhoto(0);
	    semsg.set_countsTemp(0);
	    semsg.set_tipo((short)3);
	    try {
		   //65535 emite en broadcast para todas las motas
		   mif2.send(65535,semsg);
	    } catch (IOException e) {
		  System.err.println("No se ha podido enviar el mensaje.");
	    }
	    mando.registrarMensaje("(" + getFechaHora() + ") " + men);
	}
	
	//Este método nos retorna la fecha y la hora actual en una cadena 
	//con el formato requerido
	public String getFechaHora() {
	    Calendar calendario = Calendar.getInstance();
	    SimpleDateFormat sdf = new SimpleDateFormat(FORMATO_FECHA_HORA);
	    return sdf.format(calendario.getTime());

	}
	
	//Retorna una instancia de TinyDomus
	public static TinyDomus getTinyDomus() {
		return mensajes;
	}
	
}